#include "para.h"
extern void clog(int ll, char* fmt,...);

/*
 * Init sockaddr. If he is null (server mode), use own IP, else use
 * that given in he.
 */
void init_sockaddr(struct sockaddr_in *addr, int port, struct hostent *he)
{
	/* host byte order */
	addr->sin_family = AF_INET;
	/* short, network byte order */
	addr->sin_port = htons(port);
	if (he)
		addr->sin_addr = *((struct in_addr *)he->h_addr);
	else
		addr->sin_addr.s_addr = INADDR_ANY;
	/* zero the rest of the struct */
	memset(&addr->sin_zero, '\0', 8);

}

/*
 * Taken from Beej's Guide to Network Programming
 *
 * Due to circumstances beyond your control, the kernel might not send
 * all the data out in one chunk, and now, my friend, it's up to us to
 * get the data out there.
 */

int sendall(int fd, char *buf, size_t *len)
{
	int total = 0;        /* how many bytes we've sent */
	int bytesleft = *len; /* how many we have left to send */
	int n;

	while (total < *len) {
		n = send(fd, buf + total, bytesleft, 0);
		if (n == -1)
			break;
		total += n;
		bytesleft -= n;
	}
	*len = total; /* return number actually sent here */
	return n == -1? -1 : 0; /* return -1 on failure, 0 on success */
}

/*
 * Send buffer of length n to socket. Log errors with givel loglevel
 */
int send_bin_buffer_ll(int socket_fd, char *buf, ssize_t n, int loglevel)
{
	int ret;

	ret = sendall(socket_fd, buf, &n);
	if (ret < 0)
		clog(loglevel, "%s: failed to send buffer (%s)\n", __func__,
		strerror(errno));
	return ret;
}

/*
 * Send buffer of length n to socket. Log errors with loglevel ERROR
 */
int send_bin_buffer(int socket_fd, char *buf, ssize_t n)
{
	return send_bin_buffer_ll(socket_fd, buf, n, ERROR);
}

/*
 * Send null terminated buffer to socket. Log errors with givel loglevel
 */
int send_buffer_ll(int socket_fd, char *buf, int loglevel)
{
	return send_bin_buffer_ll(socket_fd, buf, strlen(buf), ERROR);
}

/*
 * Send null terminated buffer to socket. Log errors with loglevel ERROR
 */
int send_buffer(int socket_fd, char *buf)
{
	return send_buffer_ll(socket_fd, buf, ERROR);
}

int recv_bin_buffer(int socket_fd, char *buf, ssize_t len)
{

	int numbytes;

	if ((numbytes = recv(socket_fd, buf, len - 1, 0)) == -1)
		clog(ERROR, "%s: recv error (%s)\n", __func__,
			strerror(errno));
	return numbytes;
}

int recv_buffer(int socket_fd, char *buf, ssize_t len)
{
	int n;

	if ((n = recv_bin_buffer(socket_fd, buf, len)) >= 0)
		buf[n] = '\0';
	return n;
}

struct hostent *get_host_info(char *host)
{
	struct hostent *he;

	if (!(he = gethostbyname(host)))
	clog(ERROR, "%s: gethostbyname error (%s)\n", __func__,
			strerror(errno));
	return he;

}

int get_socket(void)
{
	int socket_fd;

	if ((socket_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
		clog(ERROR, "%s: Socket error (%s)\n", __func__,
			strerror(errno));
	return socket_fd;
}

int do_connect(int socket_fd, struct sockaddr_in *their_addr)
{
	int ret;

	if ((ret = connect(socket_fd, (struct sockaddr *)their_addr,
			sizeof(struct sockaddr))) == -1)
		clog(ERROR, "%s: Connect error (%s)\n", __func__,
			strerror(errno));
	return ret;
}

int do_accept(int socket_fd, struct sockaddr_in *their_addr)
{
	int new_fd, sin_size = sizeof(struct sockaddr_in);

	if ((new_fd = accept(socket_fd, (struct sockaddr *) their_addr,
			&sin_size)) == -1)
		clog(WARNING, "%s: accept failed (caught SIGNAL?)\n",
			__func__);
	return new_fd;
}
int setserversockopts(int socket_fd)
{
	int yes = 1;

	if (setsockopt(socket_fd, SOL_SOCKET, SO_REUSEADDR, &yes,
			sizeof(int)) == -1) {
		clog(ERROR, "%s: setsockopt failed\n", __func__);
		return -1;
	}
	return 0;
}
int do_bind(int socket_fd, struct sockaddr_in *my_addr)
{
	if (bind(socket_fd, (struct sockaddr *)my_addr,
		sizeof(struct sockaddr)) == -1) {
		clog(EMERG, "%s: bind error: %s\n", __func__, strerror(errno));
		return -1;
	}
	return 0;

}
